@@ -1279,6 +1279,295 @@ layer_conv_lstm_2d <- function(object, filters, kernel_size, strides = c(1L, 1L)
 }
 
 
+#' 1D Convolutional LSTM
+#'
+#' @details
+#' Similar to an LSTM layer, but the input transformations
+#' and recurrent transformations are both convolutional.
+#'
+#' @inheritParams layer_dense
+#'
+#' @param filters Integer, the dimensionality of the output space (i.e. the number of
+#' output filters in the convolution).
+#'
+#' @param kernel_size An integer or list of n integers, specifying the
+#' dimensions of the convolution window.
+#'
+#' @param strides An integer or list of n integers, specifying the strides of
+#' the convolution. Specifying any stride value != 1 is incompatible with
+#' specifying any `dilation_rate` value != 1.
+#'
+#' @param padding One of `"valid"` or `"same"` (case-insensitive). `"valid"` means no
+#' padding. `"same"` results in padding evenly to the left/right or up/down
+#' of the input such that output has the same height/width dimension as the
+#' input.
+#'
+#' @param data_format A string, one of `channels_last` (default) or `channels_first`.
+#' The ordering of the dimensions in the inputs. `channels_last` corresponds
+#' to inputs with shape `(batch, time, ..., channels)` while `channels_first`
+#' corresponds to inputs with shape `(batch, time, channels, ...)`. It
+#' defaults to the `image_data_format` value found in your Keras config file
+#' at `~/.keras/keras.json`. If you never set it, then it will be
+#' "channels_last".
+#'
+#' @param dilation_rate An integer or list of n integers, specifying the
+#' dilation rate to use for dilated convolution. Currently, specifying any
+#' `dilation_rate` value != 1 is incompatible with specifying any `strides`
+#' value != 1.
+#'
+#' @param activation Activation function to use. By default hyperbolic tangent
+#' activation function is applied (`tanh(x)`).
+#'
+#' @param recurrent_activation Activation function to use for the recurrent step.
+#'
+#' @param use_bias Boolean, whether the layer uses a bias vector.
+#'
+#' @param kernel_initializer Initializer for the `kernel` weights matrix, used for
+#' the linear transformation of the inputs.
+#'
+#' @param recurrent_initializer Initializer for the `recurrent_kernel` weights
+#' matrix, used for the linear transformation of the recurrent state.
+#'
+#' @param bias_initializer Initializer for the bias vector.
+#'
+#' @param unit_forget_bias Boolean. If TRUE, add 1 to the bias of the forget gate at
+#' initialization. Use in combination with `bias_initializer="zeros"`. This
+#' is recommended in [Jozefowicz et al., 2015](
+#'   https://www.jmlr.org/proceedings/papers/v37/jozefowicz15.pdf)
+#'
+#' @param kernel_regularizer Regularizer function applied to the `kernel` weights
+#' matrix.
+#'
+#' @param recurrent_regularizer Regularizer function applied to the
+#' `recurrent_kernel` weights matrix.
+#'
+#' @param bias_regularizer Regularizer function applied to the bias vector.
+#'
+#' @param activity_regularizer Regularizer function applied to.
+#'
+#' @param kernel_constraint Constraint function applied to the `kernel` weights
+#' matrix.
+#'
+#' @param recurrent_constraint Constraint function applied to the `recurrent_kernel`
+#' weights matrix.
+#'
+#' @param bias_constraint Constraint function applied to the bias vector.
+#'
+#' @param return_sequences Boolean. Whether to return the last output in the output
+#' sequence, or the full sequence. (default FALSE)
+#'
+#' @param return_state Boolean Whether to return the last state in addition to the
+#' output. (default FALSE)
+#'
+#' @param go_backwards Boolean (default FALSE). If TRUE, process the input sequence
+#' backwards.
+#'
+#' @param stateful Boolean (default FALSE). If TRUE, the last state for each sample
+#' at index i in a batch will be used as initial state for the sample of
+#' index i in the following batch.
+#'
+#' @param dropout Float between 0 and 1. Fraction of the units to drop for the linear
+#' transformation of the inputs.
+#'
+#' @param recurrent_dropout Float between 0 and 1. Fraction of the units to drop for
+#' the linear transformation of the recurrent state.
+#' @param ... standard layer arguments.
+#'
+#' @seealso
+#'   +  <https://www.tensorflow.org/api_docs/python/tf/keras/layers/ConvLSTM1D>
+#'
+#' @export
+layer_conv_lstm_1d <-
+function(object,
+         filters,
+         kernel_size,
+         strides = 1L,
+         padding = "valid",
+         data_format = NULL,
+         dilation_rate = 1L,
+         activation = "tanh",
+         recurrent_activation = "hard_sigmoid",
+         use_bias = TRUE,
+         kernel_initializer = "glorot_uniform",
+         recurrent_initializer = "orthogonal",
+         bias_initializer = "zeros",
+         unit_forget_bias = TRUE,
+         kernel_regularizer = NULL,
+         recurrent_regularizer = NULL,
+         bias_regularizer = NULL,
+         activity_regularizer = NULL,
+         kernel_constraint = NULL,
+         recurrent_constraint = NULL,
+         bias_constraint = NULL,
+         return_sequences = FALSE,
+         return_state = FALSE,
+         go_backwards = FALSE,
+         stateful = FALSE,
+         dropout = 0,
+         recurrent_dropout = 0,
+         ...)
+{
+  require_tf_version("2.6", "layer_conv_lstm_1d")
+  args <- capture_args(
+    match.call(),
+    list(
+      filters = as.integer,
+      kernel_size = as_integer_tuple,
+      strides = as_integer_tuple,
+      dilation_rate = as.integer,
+      batch_size = as_nullable_integer,
+      input_shape = normalize_shape
+    ),
+    ignore = "object"
+  )
+  create_layer(keras$layers$ConvLSTM1D, object, args)
+}
+
+
+#' 3D Convolutional LSTM
+#'
+#' @details
+#' Similar to an LSTM layer, but the input transformations
+#' and recurrent transformations are both convolutional.
+#'
+#' @inheritParams layer_dense
+#'
+#' @param filters Integer, the dimensionality of the output space (i.e. the number of
+#' output filters in the convolution).
+#'
+#' @param kernel_size An integer or list of n integers, specifying the
+#' dimensions of the convolution window.
+#'
+#' @param strides An integer or list of n integers, specifying the strides of
+#' the convolution. Specifying any stride value != 1 is incompatible with
+#' specifying any `dilation_rate` value != 1.
+#'
+#' @param padding One of `"valid"` or `"same"` (case-insensitive). `"valid"` means no
+#' padding. `"same"` results in padding evenly to the left/right or up/down
+#' of the input such that output has the same height/width dimension as the
+#' input.
+#'
+#' @param data_format A string, one of `channels_last` (default) or `channels_first`.
+#' The ordering of the dimensions in the inputs. `channels_last` corresponds
+#' to inputs with shape `(batch, time, ..., channels)` while `channels_first`
+#' corresponds to inputs with shape `(batch, time, channels, ...)`. It
+#' defaults to the `image_data_format` value found in your Keras config file
+#' at `~/.keras/keras.json`. If you never set it, then it will be
+#' "channels_last".
+#'
+#' @param dilation_rate An integer or list of n integers, specifying the
+#' dilation rate to use for dilated convolution. Currently, specifying any
+#' `dilation_rate` value != 1 is incompatible with specifying any `strides`
+#' value != 1.
+#'
+#' @param activation Activation function to use. By default hyperbolic tangent
+#' activation function is applied (`tanh(x)`).
+#'
+#' @param recurrent_activation Activation function to use for the recurrent step.
+#'
+#' @param use_bias Boolean, whether the layer uses a bias vector.
+#'
+#' @param kernel_initializer Initializer for the `kernel` weights matrix, used for
+#' the linear transformation of the inputs.
+#'
+#' @param recurrent_initializer Initializer for the `recurrent_kernel` weights
+#' matrix, used for the linear transformation of the recurrent state.
+#'
+#' @param bias_initializer Initializer for the bias vector.
+#'
+#' @param unit_forget_bias Boolean. If TRUE, add 1 to the bias of the forget gate at
+#' initialization. Use in combination with `bias_initializer="zeros"`. This
+#' is recommended in [Jozefowicz et al., 2015](
+#'   https://www.jmlr.org/proceedings/papers/v37/jozefowicz15.pdf)
+#'
+#' @param kernel_regularizer Regularizer function applied to the `kernel` weights
+#' matrix.
+#'
+#' @param recurrent_regularizer Regularizer function applied to the
+#' `recurrent_kernel` weights matrix.
+#'
+#' @param bias_regularizer Regularizer function applied to the bias vector.
+#'
+#' @param activity_regularizer Regularizer function applied to.
+#'
+#' @param kernel_constraint Constraint function applied to the `kernel` weights
+#' matrix.
+#'
+#' @param recurrent_constraint Constraint function applied to the `recurrent_kernel`
+#' weights matrix.
+#'
+#' @param bias_constraint Constraint function applied to the bias vector.
+#'
+#' @param return_sequences Boolean. Whether to return the last output in the output
+#' sequence, or the full sequence. (default FALSE)
+#'
+#' @param return_state Boolean Whether to return the last state in addition to the
+#' output. (default FALSE)
+#'
+#' @param go_backwards Boolean (default FALSE). If TRUE, process the input sequence
+#' backwards.
+#'
+#' @param stateful Boolean (default FALSE). If TRUE, the last state for each sample
+#' at index i in a batch will be used as initial state for the sample of
+#' index i in the following batch.
+#'
+#' @param dropout Float between 0 and 1. Fraction of the units to drop for the linear
+#' transformation of the inputs.
+#'
+#' @param recurrent_dropout Float between 0 and 1. Fraction of the units to drop for
+#' the linear transformation of the recurrent state.
+#' @param ... standard layer arguments.
+#'
+#' @seealso
+#'   +  <https://www.tensorflow.org/api_docs/python/tf/keras/layers/ConvLSTM3D>
+#'
+#' @export
+layer_conv_lstm_3d <-
+function(object,
+         filters,
+         kernel_size,
+         strides = c(1L, 1L, 1L),
+         padding = "valid",
+         data_format = NULL,
+         dilation_rate = c(1L, 1L, 1L),
+         activation = "tanh",
+         recurrent_activation = "hard_sigmoid",
+         use_bias = TRUE,
+         kernel_initializer = "glorot_uniform",
+         recurrent_initializer = "orthogonal",
+         bias_initializer = "zeros",
+         unit_forget_bias = TRUE,
+         kernel_regularizer = NULL,
+         recurrent_regularizer = NULL,
+         bias_regularizer = NULL,
+         activity_regularizer = NULL,
+         kernel_constraint = NULL,
+         recurrent_constraint = NULL,
+         bias_constraint = NULL,
+         return_sequences = FALSE,
+         return_state = FALSE,
+         go_backwards = FALSE,
+         stateful = FALSE,
+         dropout = 0,
+         recurrent_dropout = 0,
+         ...)
+{
+  require_tf_version("2.6", "layer_conv_lstm_3d")
+  args <- capture_args(
+    match.call(),
+    list(
+      filters = as.integer,
+      kernel_size = as_integer_tuple,
+      strides = as_integer_tuple,
+      dilation_rate = as.integer,
+      batch_size = as_nullable_integer,
+      input_shape = normalize_shape
+    ),
+    ignore = "object"
+  )
+  create_layer(keras$layers$ConvLSTM3D, object, args)
+}
+
 
 
 
